#########################################################################
##  GMM.R
##  
##  This is a R program, written by
##  Bruce E. Hansen
##  Department of Economics
##  University of Wisconsin
##  behansen@wisc.edu
##  http:/www.ssc.wisc.edu/~bhansen
##  
#########################################################################

trm <- .02       # Percentile of high-income households deleted 

#********************************************************#

smp <- 1         # Set to 1 if "urban" or 2 if "rural"  

while (smp<=2){    

rname <- rbind("Region1","Region2","Region3","Region4","Region5",
               "Region6","Region7","Region8","Region9","Region10",
               "Region11","Region12","Region13")

vn <- rbind("Abroad  ","Domestic","Inkind  ","Gifts   ","Income  ",                    
            "Wages   ","Self    ","Rent    ","Interest","RetInc  ",                     
            "Dividend","Primary ","Some Hgh","High Grd","Some Col",                     
            "College ","Age     ","Female  ","Married ","Kids<1  ",                     
            "1<Kids<7","7<Kid<15","HH Size ","Both Wrk","Not Empl",                    
            "Mar Fem ","Region  ")

if (smp==1){
  rname <- rbind("Region0",rname)
  sample <- "Urban"
  dat <- read.table("urban.dat")
  knot <- 21840
}
if (smp==2){ 
  sample <- "Rural"
  dat <- read.table("rural.dat")
  knot <- 10793
}

abroad <- dat[,1]
domestic <- dat[,2]
inkind <- dat[,3]
gifts <- dat[,4]
totinc <- dat[,5]
retinc <- dat[,10]
yname <- "Transfers"

y <- abroad+domestic+inkind-gifts
i <- totinc+gifts-abroad-domestic-inkind-retinc

# delete observations with negative income #
dat <- cbind(dat,i,y)
indx <- (i>=0)
k <- ncol(dat)
dat <- as.matrix(dat[indx%*%matrix(1,1,k)>0])
dat <- matrix(dat,nrow(dat)/k,k)

i <- sort(dat[,28])
n <- floor(nrow(dat)*(1-trm))

indx <- as.matrix((dat[,28]<=i[n]))
k <- ncol(dat)
dat <- as.matrix(dat[indx%*%matrix(1,1,k)>0])
dat <- matrix(dat,nrow(dat)/k,k)

retinc <- dat[,10]
i <- as.matrix(dat[,28])
y <- as.matrix(dat[,29])
dat <- dat[,1:27]

dat[,23] <- dat[,23]-dat[,20]-dat[,21]-dat[,22]
vn[23] <- "Adults"
xx <- cbind((retinc>0),retinc,dat[,12:17])
xn <- rbind("Has Retr",vn[10],as.matrix(vn[12:17]))

region <- as.matrix(dat[,27])
r <- unique(region)
r <- as.matrix(sort(r))
rr <- nrow(r)
xreg <- (region%*%matrix(1,1,nrow(r))==matrix(1,nrow(region),1)%*%t(r))
mx <- matrix(1,n,1)%*%t(as.matrix(colMeans(xx)))

n <- nrow(dat)
x1 <- cbind(xx,dat[,18:26],matrix(1,n,1))      # regressors for reduced form
x2 <- cbind(xx,dat[,18:26],xreg)		     # regressors for main equation
xn <- rbind(xn,as.matrix(vn[18:26]),rname)     # names of regressors

# Reduced form for endogenous income #
rs <- as.matrix(xreg[,1])
ir <- as.matrix(i[rs])
xr <- x1[rs%*%matrix(1,1,ncol(x1))>0]
xr <- matrix(xr,length(xr)/ncol(x1),ncol(x1))
ihat <- xr%*%qr.solve(xr,ir)
for (j in 2:rr){
    rs <- as.matrix(xreg[,j])
    ir <- as.matrix(i[rs])
    xr <- x1[rs%*%matrix(1,1,ncol(x1))>0]
    xr <- matrix(xr,length(xr)/ncol(x1),ncol(x1))
    ih <- xr%*%qr.solve(xr,ir)
    ihat <- rbind(ihat,ih)
}
z <- cbind(ihat,(ihat^2),x2)   # instruments #

cat ("************************************************************","\n")
cat ("\n")
cat ("\n")
cat (sample,"households","\n")
cat ("Dependent Variable:            ",yname,"\n")
cat ("Trimmed Top Quantile of Income ",trm,"\n")
cat ("Observations                   ", n,"\n")
cat ("__________________________________________________","\n")


#*************************************************#
# Linear Model #
x <- cbind(i,x2)
w <- solve(t(z)%*%z,tol=1e-040,LAPACK=FALSE) 
zx <- t(z)%*%x
xxi <- solve(t(zx)%*%w%*%zx)  
beta <- xxi%*%(t(zx)%*%w%*%(t(z)%*%y))
e <- y - x%*%beta
ze <- z*(e%*%matrix(1,1,ncol(z)))
w <- solve(t(ze)%*%ze,tol=1e-040,LAPACK=FALSE)  
xxi <- solve(t(zx)%*%w%*%zx)  
beta <- xxi%*%(t(zx)%*%w%*%(t(z)%*%y))
e <- y - x%*%beta
ze <- z*(e%*%matrix(1,1,ncol(z)))
v <- solve(t(zx)%*%solve(t(ze)%*%ze,tol=1e-040,LAPACK=FALSE)%*%zx)
se <- sqrt(diag(v))
j <- (t(e)%*%z)%*%w%*%(t(z)%*%e)
dof <- ncol(z)-ncol(x)
xname <- rbind("Income",xn)
Txname <- format(xname,digits=4)
Tbeta <- format(beta,digits=4)
Tse <- format(se,digits=4) 
cat ("Linear Model","\n")
cat ("\n")
cat ("Variable    ","Estimate    ","St Error    ","\n")
cat ("____________________________________________________","\n")
for (jj in 1:nrow(beta)) cat (Txname[jj]," ",Tbeta[jj],"  ",Tse[jj],"\n")
if (dof>0) cat ("J-Statistic, dof, pvalue ",j," ",dof," ",1-pchisq(j,dof),"\n")
cat ("____________________________________________________","\n")
cat ("\n")
cat ("\n")
cat ("\n")

# Linear Spline #
ic <- i - knot
i1 <- (i<=knot)
i2 <- 1-i1
dx <- cbind((ic*i1),(ic*i2))
x <- cbind(dx,x2)
zx <- t(z)%*%x
xxi <- solve(t(zx)%*%w%*%zx)  
beta <- xxi%*%(t(zx)%*%w%*%(t(z)%*%y))
e <- y - x%*%beta
ze <- z*(e%*%matrix(1,1,ncol(z)))
v <- solve(t(zx)%*%solve(t(ze)%*%ze,tol=1e-040,LAPACK=FALSE)%*%zx)
se <- sqrt(diag(v))
j <- (t(e)%*%z)%*%w%*%(t(z)%*%e)
dof <- ncol(z)-ncol(x)

xname <- rbind("Slope_1","Slope_2",xn)
Txname <- format(xname,digits=4)
Tbeta <- format(beta,digits=4)
Tse <- format(se,digits=4) 
cat ("Linear Spline","\n")
cat ("\n")  
cat ("Variable    ","Estimate    ","St Error    ","\n")
cat ("____________________________________________________","\n")
for (jj in 1:nrow(beta)) cat (Txname[jj]," ",Tbeta[jj],"  ",Tse[jj],"\n")
if (dof>0) cat ("J-Statistic, dof, pvalue ",j," ",dof," ",1-pchisq(j,dof),"\n")
cat ("____________________________________________________","\n")
cat ("\n")
cat ("\n")
cat ("\n")

mx <- as.matrix(colMeans(x2))
tr <- dx%*%beta[1:2]+as.vector(t(mx)%*%beta[3:nrow(beta)])
gdat <- cbind(i,tr)
indx <- order(i)
gdat <- gdat[indx,]
mtit <- paste(sample,"households")
ytit <- "Transfers (Pesos)"
xtit <- "Total Income (Pesos)"
if (smp==1){
 xtics <- cbind(0,200000) 
 ytics <- cbind(0,20000)
} 
if (smp==2){
 xtics <- cbind(0,80000)
 ytics <- cbind(0,10000)
}
x11()
plot(gdat[,1],gdat[,2],lty=1,col=1,type="l",xlim=xtics,ylim=ytics,ann=0)
title(main=mtit,ylab=ytit,xlab=xtit) 

smp <- smp+1
}


