#########################################################################
##  ROBUST.R
##  
##  This is a R program, written by
##  Bruce E. Hansen
##  Department of Economics
##  University of Wisconsin
##  behansen@wisc.edu
##  http:/www.ssc.wisc.edu/~bhansen
##  
##  This program estimates the spline transfer function model presented in
##  "How Responsive are Private Transfers to Income?
##  Evidence From a Laissez-Faire Economy"
##  by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.
##  
##  The results of this program generates the estimates reported in Table 6.
##  
##  This program will take a long time to execute if one sets qn=0.  To speed up
##  execution (with little cost in precision), set qn=500 or even qn=100.
##  
#########################################################################

trm <- .02       # Percentile of high-income households deleted  
per <- .10       # Percentage to trim from quantiles for threshold search 
qn  <- 0         # Set qn=0  to search across all thresholds.
                 # Set qn to a positive integers to search
                 # across a fixed number of quantiles     
noinc <- 1       # If set to 1, includes dummy for no income, else set to 0 

#***********************************************************#

model <- 1 
while (model<=9){
smp <- 1             # Set to 1 if "urban" or 2 if "rural"  
while (smp<=2){  

cat ("************************************************************","\n")
cat ("\n")
cat ("\n")
noinc <- 1
if (model==1){
  cat ("International Transfer Recipients Excluded","\n") 
  if (smp==1) {noinc=1 
  }else{noinc=0}
}
if (model==2) cat("Not Secondary Graduate","\n")
if (model==3) cat("Secondary Graduate","\n")
if (model==4) cat("Family Size 4 or Less","\n")
if (model==5){
  cat ("Family Size 5 or More","\n")
  if (smp==1) {noinc=1 
  }else{noinc=0}
}
if (model==6){
  cat ("Household Head Aged 45 or less","\n")
  if (smp==1) {noinc=1 
  }else{noinc=0}
}
if (model==7){
  cat ("Household Head over age 45","\n")
  if (smp==1) {noinc=1 
  }else{noinc=0}
}
if (model==8) cat("Gross Transfers Received","\n")
if (model==9) cat("Gross Transfers Given (Gifts)","\n")

rname <- rbind("Region1","Region2","Region3","Region4","Region5",
               "Region6","Region7","Region8","Region9","Region10",
               "Region11","Region12","Region13")

vn <- rbind("Abroad  ","Domestic","Inkind  ","Gifts   ","Income  ",                    
            "Wages   ","Self    ","Rent    ","Interest","RetInc  ",                     
            "Dividend","Primary ","Some Hgh","High Grd","Some Col",                     
            "College ","Age     ","Female  ","Married ","Kids<1  ",                     
            "1<Kids<7","7<Kid<15","HH Size ","Both Wrk","Not Empl",                    
            "Mar Fem ","Region  ")

if (smp==1){
  rname <- rbind("Region0",rname)
  sample <- "Urban"
  dat <- read.table("urban.dat")
}
if (smp==2){ 
  sample <- "Rural"
  dat <- read.table("rural.dat")
}

abroad <- dat[,1]
domestic <- dat[,2]
inkind <- dat[,3]
gifts <- dat[,4]
totinc <- dat[,5]
retinc <- dat[,10]
yname <- "Transfers"
i <- as.matrix(totinc+gifts-abroad-domestic-inkind-retinc)
ii <- as.matrix(sort(i))
qi <- ii[floor(nrow(dat)*(1-trm))]

dd <- (i<0)+(i>qi)
if (model==1) dd <- dd+(1-dd)*(abroad>0)
if (model==2) dd <- dd+(1-dd)*(dat[,14]+dat[,15]+dat[,16]>0)
if (model==3) dd <- dd+(1-dd)*(dat[,12]+dat[,13]>0)
if (model==4) dd <- dd+(1-dd)*(dat[,23]>4)
if (model==5) dd <- dd+(1-dd)*(dat[,23]<=4)
if (model==6) dd <- dd+(1-dd)*(dat[,17]>45)
if (model==7) dd <- dd+(1-dd)*(dat[,17]<=45)

y <- abroad+domestic+inkind-gifts
if (model==1) y <- domestic+inkind-gifts
if (model==8) y <- abroad+domestic+inkind
if (model==9) y <- gifts

dat <- cbind(dat,i,y)
indx <- (dd==0)
k <- ncol(dat)
dat <- as.matrix(dat[indx%*%matrix(1,1,k)>0])
dat <- matrix(dat,nrow(dat)/k,k)
n <- nrow(dat)
retinc <- dat[,10]
i <- as.matrix(dat[,28])
y <- as.matrix(dat[,29])

dat[,23] <- dat[,23]-dat[,20]-dat[,21]-dat[,22]
vn[23] <- "Adults"

xx <- cbind((retinc>0),retinc,dat[,12:17])
xn <- rbind("Has Retr",vn[10],as.matrix(vn[12:17]))

if (model==2){
  xx <- cbind((retinc>0),retinc,dat[,17])
  xn <- rbind("Has Retr",vn[10],vn[17])}
if (model==3){
  xx <- cbind((retinc>0),retinc,dat[,17])
  xn <- rbind("Has Retr",vn[10],vn[17])}

region <- as.matrix(dat[,27])
r <- unique(region)
r <- as.matrix(sort(r))
xreg <- (region%*%matrix(1,1,nrow(r))==matrix(1,nrow(region),1)%*%t(r))
xx <- cbind(xx,dat[,18:26],xreg)
xn <- rbind(xn,as.matrix(vn[18:26]),rname)
mx <- matrix(1,n,1)%*%t(as.matrix(colMeans(xx)))
if (noinc==1){
  i0 <- (i==0)
  xx <- cbind(i0,xx)
  xn <- rbind("No Income",xn)
}

#*************************************************#
# Linear Spline Model #

xm <- xx%*%solve(t(xx)%*%xx, tol=1e-100, LAPACK=FALSE)  
e0 <- y - xm%*%(t(xx)%*%y)
ee0 <- t(e0)%*%e0
q <- sort(i)
if (qn == 0){
  q <- as.matrix(unique(q[floor(per*n):(n-floor(per*n))]))
  qn <- nrow(q)
}else{
  q <- as.matrix(q[floor(seq(per,per+(qn-1)*(1-per*2)/qn,(1-per*2)/qn)*n)])
}

sse <- matrix(0,qn,1) 
for (j in 1:qn){
  qj <- q[j]
  ic <- i - qj
  i1 <- (i<=qj)
  i2 <- 1-i1
  dx <- cbind((ic*i1),(ic*i2))
  xv <- dx - xm%*%(t(xx)%*%dx)
  et <- e0 - xv%*%qr.solve(xv,e0)
  sse[j] <- t(et)%*%et
  if (round(j/10)==(j/10)) cat (j,qj,(sse[j]/n),"\n")
}
jmin <- which.min(sse)
knot <- q[jmin]

ic <- i - knot
i1 <- (i<=knot)
i2 <- 1-i1
dx <- cbind((ic*i1),(ic*i2))
x <- cbind(dx,xx)
beta <- qr.solve(x,y)
e <- y - x%*%beta
xh <- -beta[1]*i1-beta[2]*i2
x <- cbind(x,xh)
tr1 <- dx%*%beta[1:2]+mx%*%beta[(3+noinc):nrow(beta)]

xname <- rbind("Slope_1","Slope_2")
xname <- rbind(xname,xn,"Threshold")
theta <- rbind(beta,knot)

cat ("************************************************************","\n")
cat ("\n")
cat ("\n")
cat (sample,"households","\n")
cat ("Observations                   ", n,"\n")
cat ("__________________________________________________","\n")
  ee <- t(e)%*%e
  sig <- as.vector(ee/(nrow(y)-ncol(x)))
  mi <- solve(t(x)%*%x, tol=1e-100, LAPACK=FALSE)
  se <- as.matrix(sqrt(abs(diag(mi*sig))))
  xe <- x*(e%*%matrix(1,1,ncol(x)))
  seh <- as.matrix(sqrt(abs(diag(mi%*%(t(xe)%*%xe)%*%mi))))
  Txname <- format(xname,digits=4)
  Ttheta <- format(theta,digits=4)
  Tse <- format(se,digits=4) 
  Tseh <- format(seh,digits=4)
  xse <- cbind(Txname,Ttheta,Tse,Tseh)
  if (noinc==0){
    xse <- rbind(xse[1:2,],xse[nrow(xse),])
  }else{
    xse <- rbind(xse[1:3,],xse[nrow(xse),])
  }
  cat ("Variable    ","Estimate    ","St Error    ","Het St Er    ","\n")
  cat ("____________________________________________________","\n")
  for (j in 1:nrow(xse)) cat (xse[j,1]," ",xse[j,2],"  ",xse[j,3],"  ",xse[j,4],"\n")
  cat ("____________________________________________________","\n")
  cat ("\n")
  cat ("\n")
  cat ("\n")

smp <- smp+1
}

model <- model+1
}

