#########################################################################
## SMOOTH.R
## 
## This is a R program, written by
## Bruce E. Hansen
## Department of Economics
## University of Wisconsin
## behansen@wisc.edu
## http:/www.ssc.wisc.edu/~bhansen
## 
## This program estimates the spline transfer function model presented in
## "Are Private Transfers Altruistically Motivated?",
## by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.
## 
## This program estimates the model with random thresholds.
## 
#########################################################################

trm <- .02    # Percentile of high-income households trimmed from sample
per <- .05    # Percentage to trim from quantiles for threshold search 
qn  <- 100    # Set qn=0  to search across all thresholds.
              # Set qn to a positive integers to search across a fixed number of quantiles  
autop <- 0    # If set to 1, will automatically print graphs   


#******************* function ***************************#

gr_tr <- function(i,tr){
mtit <- paste(sample,"households")
ytit <- "Transfers (Pesos)"
xtit <- "Total Income (Pesos)"
if (smp==1){
 xtics <- cbind(0,200000) 
 ytics <- cbind(0,20000)
} 
if (smp==2){
 xtics <- cbind(0,80000)
 ytics <- cbind(0,10000)
}
x11()
plot(i,tr[,1],lty=1,col=1,type="l",xlim=xtics,ylim=ytics,ann=0)
lines(i,tr[,2],lty=2,col=2)
title(main=mtit,ylab=ytit,xlab=xtit) 
}

#********************************************************#

smp <- 1         # Set to 1 if "urban" or 2 if "rural"  

while (smp<=2){    

rname <- rbind("Region1","Region2","Region3","Region4","Region5",
               "Region6","Region7","Region8","Region9","Region10",
               "Region11","Region12","Region13")

vn <- rbind("Abroad  ","Domestic","Inkind  ","Gifts   ","Income  ",                    
            "Wages   ","Self    ","Rent    ","Interest","RetInc  ",                     
            "Dividend","Primary ","Some Hgh","High Grd","Some Col",                     
            "College ","Age     ","Female  ","Married ","Kids<1  ",                     
            "1<Kids<7","7<Kid<15","HH Size ","Both Wrk","Not Empl",                    
            "Mar Fem ","Region  ")

if (smp==1){
  rname <- rbind("Region0",rname)
  sample <- "Urban"
  dat <- read.table("urban.dat")
}
if (smp==2){ 
  sample <- "Rural"
  dat <- read.table("rural.dat")
}

abroad <- dat[,1]
domestic <- dat[,2]
inkind <- dat[,3]
gifts <- dat[,4]
totinc <- dat[,5]
retinc <- dat[,10]
yname <- "Transfers"

y <- abroad+domestic+inkind-gifts
i <- totinc+gifts-abroad-domestic-inkind-retinc

# delete observations with negative income #
dat <- cbind(dat,i,y)
indx <- (i>=0)
k <- ncol(dat)
dat <- as.matrix(dat[indx%*%matrix(1,1,k)>0])
dat <- matrix(dat,nrow(dat)/k,k)

indx <- order(dat[,28])
dat <- dat[indx,] 
n <- floor(nrow(dat)*(1-trm))
dat <- dat[1:n,]
retinc <- dat[,10]
i <- as.matrix(dat[,28])
y <- as.matrix(dat[,29])
dat <- dat[,1:27]

dat[,23] <- dat[,23]-dat[,20]-dat[,21]-dat[,22]
vn[23] <- "Adults"
xx <- cbind((retinc>0),retinc,dat[,12:17])
xn <- rbind("Has Retr",vn[10],as.matrix(vn[12:17]))
region <- as.matrix(dat[,27])
r <- unique(region)
r <- as.matrix(sort(r))
xreg <- (region%*%matrix(1,1,nrow(r))==matrix(1,nrow(region),1)%*%t(r))
xx <- cbind(xx,dat[,18:26],xreg)
xn <- rbind(xn,as.matrix(vn[18:26]),rname)
mx <- matrix(1,n,1)%*%t(as.matrix(colMeans(xx)))
i0 <- (i==0)
xx <- cbind(i0,xx)
xn <- rbind("No Income",xn)

cat ("************************************************************","\n")
cat ("\n")
cat ("\n")
cat ("Dependent Variable:            ",yname,"\n")
cat (sample,"households","\n")
cat ("Trimmed Top Quantile of Income ",trm,"\n")
cat ("Number of Zero Income          ",sum(i0),"\n")
cat ("Observations                   ", n,"\n")
cat ("__________________________________________________","\n")
cat ("\n")

#*************************************************#

xm <- xx%*%solve(t(xx)%*%xx)  
e0 <- y - xm%*%(t(xx)%*%y)
ee0 <- t(e0)%*%e0
if (qn == 0){
  q <- sort(i)
  q <- as.matrix(unique(q[floor(per*n):(n-floor(per*n))]))
  qn <- nrow(q)
}else{
  q <- sort(i)
  q <- as.matrix(q[floor(seq(per,per+(qn-1)*(1-per*2)/qn,(1-per*2)/qn)*n)])
}

#*************************************************#

# Continuous Linear Spline Model (for starting values) #
sse <- matrix(0,qn,1) 
for (j in 1:qn){
  qj <- q[j]
  ic <- i - qj
  i1 <- (i<=qj)
  i2 <- 1-i1
  dx <- cbind((ic*i1),(ic*i2))
  xv <- dx - xm%*%(t(xx)%*%dx)
  et <- e0 - xv%*%qr.solve(xv,e0)
  sse[j] <- t(et)%*%et
  if (round(j/10)==(j/10)) cat (j,qj,(sse[j]/n),"\n")
}
jmin <- which.min(sse)
qhat <- q[jmin]

ic <- i - qhat
i1 <- (i<=qhat)
i2 <- 1-i1
dx <- cbind((ic*i1),(ic*i2))
x  <- cbind(dx,xx)
beta <- qr.solve(x,y)
e <- y - x%*%beta
ee <- t(e)%*%e

ba <- beta[3:nrow(beta)]
ya <- y - xx%*%ba
tr1 <- dx%*%beta[1:2]+mx%*%beta[4:nrow(beta)]


#*************************************************#

# Smooth Model - Step 1 #

ss <- function(theta){
  ik <- i-theta[1]
  omega <- abs(theta[2])
  a <- ik/omega
  cdfa <- pnorm(a)
  dx <- (ik%*%matrix(1,1,2))*(cbind((1-cdfa),cdfa))+(dnorm(a)%*%cbind(-1,1))*omega
  e <- ya-dx%*%qr.solve(dx,ya)
  (t(e)%*%(e)-ee)/nrow(e)
}

theta0 <- cbind(rbind(qhat,1000),rbind((qhat-2000),1000),
                rbind(qhat,5000),rbind((qhat-2000),8000))  
tn <- ncol(theta0)
theta <- matrix(0,2,tn) 
smin <- matrix(0,tn,1) 

for (ti in 1:tn){
  out <- optim(theta0[,ti],ss)
  theta[,ti] <- as.matrix(out$par)
  smin[ti] <- out$value   
}
theta1 <- as.matrix(theta[,which.min(smin)])

# Smooth Model - Step 2 #

s_smooth <- function(theta){
  ik <- i-theta[1]
  omega <- abs(theta[2])
  a <- ik/omega
  cdfa <- pnorm(a)
  dx <- (ik%*%matrix(1,1,2))*(cbind((1-cdfa),cdfa))+(dnorm(a)%*%cbind(-1,1))*omega
  xv <- dx - xm%*%(t(xx)%*%dx)
  e  <- e0 - xv%*%qr.solve(xv,e0)
  (t(e)%*%e-ee)/nrow(e)
}

out <- optim(theta1,s_smooth)
theta <- as.matrix(out$par)
  
knot <- theta[1]
omega <- abs(theta[2])
ik <- i-knot
a <- ik/omega
cdfa <- pnorm(a)
pdfa <- dnorm(a)
dx <- (ik%*%matrix(1,1,2))*(cbind((1-cdfa),cdfa))+(pdfa%*%cbind(-1,1))*omega
x  <- cbind(dx,xx)
beta <- qr.solve(x,y)
e <- y - x%*%beta
ee <- t(e)%*%e
k <- ncol(x)
sig <- (t(e)%*%(e))/(n-k-2)
r_2 <- 1 - ee/(sum((y-mean(y))^2))
theta <- rbind(knot,omega,beta)

if (omega>10){
b1 <- beta[1]
b2 <- beta[2]
b12 <- b1-b2
g1 <- -((1-cdfa)*b1+cdfa*b2)/omega+a*pdfa*b12*(1-omega)/omega
g2 <- -((1-cdfa)*b1+cdfa*b2)*a/omega-pdfa*b12+pdfa*(a^2)*(b12*(1-omega)/omega)
gx <- cbind(g1,g2,x)
h <- solve(t(gx)%*%gx)  
se <- as.matrix(sqrt(diag(h)*sig))
gxe <- gx*(e%*%matrix(1,1,ncol(gx)))
seh <- as.matrix(sqrt(diag(h%*%(t(gxe)%*%gxe)%*%h)))
}else{ 
se <- matrix(0,nrow(theta),1)  
seh <- se
}

xname <- rbind("Knot","Omega","Slope_1","Slope_2",xn)
Txname <- format(xname,digits=4)
Ttheta <- format(theta,digits=4)
Tse <- format(se,digits=4) 
Tseh <- format(seh,digits=4)
cat ("\n")
cat ("Smooth Spline Model", "\n")
cat ("Sum of Squared Errors               ", ee, "\n")
cat ("Residual Standard Deviation         ", sqrt(sig), "\n")
cat ("R-squared                           ", r_2, "\n")
cat ("\n")
cat ("Variable    ","Estimate    ","St Error    ","Het St Er    ","\n")
cat ("____________________________________________________","\n")
for (j in 1:nrow(theta)) cat (Txname[j]," ",Ttheta[j],"  ",Tse[j],"  ",Tseh[j],"\n")
cat ("____________________________________________________","\n")
cat ("\n")
cat ("\n")
cat ("\n")

mx <- matrix(1,n,1)%*%t(as.matrix(colMeans(xx)))  
xr <- cbind(dx,mx)
tr3 <- xr%*%beta
out <- gr_tr(i,cbind(tr1,tr3))

smp <- smp+1
}



