#########################################################################
##  TEST.R
##
##  This is a R program, written by
##  Bruce E. Hansen, Department of Economics, University of Wisconsin
##  behansen@wisc.edu
##  http:/www.ssc.wisc.edu/~bhansen
##
##  This program tests the spline transfer function model presented in
##  "Are Private Transfers Altruistically Motivated?",
##  by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.
## 
##  This program tests the spline model against polynomial alternatives.
##  
##  The results create Table 4 from the paper.
##  
#########################################################################

trm <- .02    # Percentile of high-income households trimmed from sample
qn  <- 100    # number of quantiles used for test. We set qn=100 
rep <- 1000   # Number of bootstrap replications.  
noinc <- 1    # If set to 1, includes dummy for no income 


#******************* function ***************************#

test <- function(y,xx,i,q,p){
  sp <- t(as.matrix(seq(1,p,1)))
  x <- cbind((i%*%matrix(1,1,p))^(matrix(1,nrow(i),1)%*%sp),xx)
  xm <- x%*%solve(t(x)%*%x, tol=1e-100, LAPACK=FALSE)  
  e <- y - xm%*%(t(x)%*%y)
  ee0 <- t(e)%*%e
  sse <- matrix(0,qn,1) 
  for (j in 1:qn){
    knot <- q[j]
    dx <- (((i-knot)%*%matrix(1,1,p))^(matrix(1,nrow(i),1)%*%sp))*((i<=knot)%*%matrix(1,1,p))
    xv <- dx - xm%*%(t(x)%*%dx)
    et <- e - xv%*%div(e,xv)
    sse[j] <- t(et)%*%et
  }
  j <- which.min(sse)
  knot <- q[j]
  ssemin <- sse[j]
  lr <- as.vector((ee0-ssemin)/ssemin*nrow(y))
  dx <- (((i-knot)%*%matrix(1,1,p))^(matrix(1,nrow(i),1)%*%sp))*((i<=knot)%*%matrix(1,1,p))
  xv <- dx - xm%*%(t(x)%*%dx)
  et <- e - xv%*%div(e,xv)
  list(lr=lr,et=et)
}

div <- function(e,x){
  if (qr(x)$rank==ncol(x)) beta <- qr.solve(x,e)
  if (qr(x)$rank<ncol(x)) beta <- (qr(t(x)%*%x)$qr)%*%(t(x)%*%e)
  if (ncol(beta)>1) beta <- beta[,1]
beta
}

#********************************************************#


smp <- 1         # Set to 1 if "urban" or 2 if "rural"  
while (smp<=2){    

rname <- rbind("Region1","Region2","Region3","Region4","Region5",
               "Region6","Region7","Region8","Region9","Region10",
               "Region11","Region12","Region13")
if (smp==1){
  rname <- rbind("Region0",rname)
  sample <- "Urban"
  dat <- read.table("urban.dat")
}
if (smp==2){ 
  sample <- "Rural"
  dat <- read.table("rural.dat")
}

cat (sample,"households","\n")
cat ("Trimmed Top Quantile of Income ", trm, "\n")
cat ("Bootstrap Replications         ", rep, "\n")
cat ("Number of Quantiles            ", qn, "\n")
cat ("Tests for a Single Knot","\n")
cat ("\n")
cat ("     ","P","     ","F-Test","     ","P-Value", "\n")

abroad <- dat[,1]
domestic <- dat[,2]
inkind <- dat[,3]
gifts <- dat[,4]
totinc <- dat[,5]
retinc <- dat[,10]
yname <- "Transfers"

y <- abroad+domestic+inkind-gifts
i <- totinc+gifts-abroad-domestic-inkind-retinc

# delete observations with negative income #
dat <- cbind(dat,i,y)
indx <- (i>=0)
k <- ncol(dat)
dat <- as.matrix(dat[indx%*%matrix(1,1,k)>0])
dat <- matrix(dat,nrow(dat)/k,k)

indx <- order(dat[,28])
dat <- dat[indx,] 
n <- floor(nrow(dat)*(1-trm))
dat <- dat[1:n,]
retinc <- dat[,10]
i <- as.matrix(dat[,28])
y <- as.matrix(dat[,29])
dat <- dat[,1:27]

dat[,23] <- dat[,23]-dat[,20]-dat[,21]-dat[,22]
xx <- cbind((retinc>0),retinc,dat[,12:17])
region <- as.matrix(dat[,27])
r <- unique(region)
r <- as.matrix(sort(r))
xreg <- (region%*%matrix(1,1,nrow(r))==matrix(1,nrow(region),1)%*%t(r))
xx <- cbind(xx,dat[,18:26],xreg)
mx <- matrix(1,n,1)%*%t(as.matrix(colMeans(xx)))
if (noinc==1){
  i0 <- (i==0)
  xx <- cbind(i0,xx)
}


p <- 1   # Order of polynomial in splined sections #
while (p<=7){       

#if (p<=4) per <- .05
#if (p<=6&p>4) per <- .15
#if (p>6) per <- .3
per <- .05
q <- sort(i)
q <- as.matrix(q[floor(seq(per,per+(qn-1)*(1-per*2)/qn,(1-per*2)/qn)*n)])
out <- test(y,xx,i,q,p)
lr <- out$lr
e <- out$et

# Bootstrap  #
boot <- matrix(0,rep,1) 
for (r in 1:rep){
  yb <- as.matrix(e[ceiling(runif(n)*n)])
  out <- test(yb,xx,i,q,p)
  boot[r] <- out$lr
}
pv <- mean(boot>=lr)

cat ("     ", p ,"     ",format(lr,digits=6) ,"     ",pv,"\n")

p <- p+1
}
cat ("\n")
cat ("\n")
cat ("\n")
smp <- smp+1
}




