#########################################################################
##  TEST2.R
##  
##  This is a R program, written by
##  Bruce E. Hansen, Department of Economics, University of Wisconsin
##  behansen@wisc.edu
##  http:/www.ssc.wisc.edu/~bhansen
##  
##  This program tests the single-knot specification against a two-knot
##  alternative.
##  
##  This results generate Table 5 from the paper.
##  
#########################################################################

trm <- .02      # Percentile of high-income households deleted  
rep <- 1        # Number of bootstrap replications.  
per <- .05      # Percentage to trim from quantiles for threshold search 
qn  <- 1000     # Set qn to a positive integers to search
                # across a fixed number of quantiles     
noinc <- 1      # If set to 1, includes dummy for no income, else set to 0


#******************* function ***************************#

test2 <- function(y,xx,i,q){
  xm <- xx%*%solve(t(xx)%*%xx, tol=1e-100, LAPACK=FALSE)
  e <- y - xm%*%(t(xx)%*%y)
  ee0 <- t(e)%*%e
  sse <- matrix(0,qn,1)
  for (j1 in 1:qn){ 
    k1 <- q[j1]
    i1 <- (i<k1)
    i2 <- 1-i1
    x1 <- (i-k1)*i1
    x2 <- (i-k1)*i2
    dx <- cbind(x1,x2)
    xv <- dx - xm%*%(t(xx)%*%dx)
    et <- e - xv%*%div(e,xv)
    sse[j1] <- t(et)%*%et
  }
  j1 <- which.min(sse)
  k1 <- q[j1]
  sse0 <- sse[j1]

  i1 <- (i<k1)
  x1 <- i*i1+(1-i1)*k1
  sse <- matrix(0,qn,1)
  for (j2 in 1:qn){ 
    if (abs(j1-j2)>=ceiling(per*qn)){
      k2 <- q[j2]
      i2 <- (i<k2)-i1
      i3 <- 1-i1-i2
      x2 <- (i-k1)*i2+i3*(k2-k1)
      x3 <- (i-k2)*i3
      dx <- cbind(x1,x2,x3)
      xv <- dx - xm%*%(t(xx)%*%dx)
      et <- e - xv%*%div(e,xv)
      sse[j2] <- t(et)%*%et-sse0
    }
  }
  j2 <- which.min(sse)
  k2 <- q[j2]
  sse1 <- sse[j2]+sse0
  f <- (sse0-sse1)/sse1*n
  i2 <- (i<k2)-i1
  i3 <- 1-i1-i2
  x2 <- (i-k1)*i2+i3*(k2-k1)
  x3 <- (i-k2)*i3
  dx <- cbind(x1,x2,x3)
  xv <- dx - xm%*%(t(xx)%*%dx)
  beta <- div(e,xv)
  et <- e - xv%*%beta
  yp <- dx%*%beta
  list(f=f,yp=yp,et=et)
}

div <- function(e,x){
  if (qr(x)$rank==ncol(x)) beta <- qr.solve(x,e)
  if (qr(x)$rank<ncol(x)) beta <- (qr(t(x)%*%x)$qr)%*%(t(x)%*%e)
  if (ncol(beta)>1) beta <- beta[,1]
beta
}

#********************************************************#


smp <- 1         # Set to 1 if "urban" or 2 if "rural"  
while (smp<=2){    

rname <- rbind("Region1","Region2","Region3","Region4","Region5",
               "Region6","Region7","Region8","Region9","Region10",
               "Region11","Region12","Region13")
if (smp==1){
  rname <- rbind("Region0",rname)
  sample <- "Urban"
  dat <- read.table("urban.dat")
}
if (smp==2){ 
  sample <- "Rural"
  dat <- read.table("rural.dat")
}

abroad <- dat[,1]
domestic <- dat[,2]
inkind <- dat[,3]
gifts <- dat[,4]
totinc <- dat[,5]
retinc <- dat[,10]
yname <- "Transfers"

y <- abroad+domestic+inkind-gifts
i <- totinc+gifts-abroad-domestic-inkind-retinc

# delete observations with negative income #
dat <- cbind(dat,i,y)
indx <- (i>=0)
k <- ncol(dat)
dat <- as.matrix(dat[indx%*%matrix(1,1,k)>0])
dat <- matrix(dat,nrow(dat)/k,k)

indx <- order(dat[,28])
dat <- dat[indx,] 
n <- floor(nrow(dat)*(1-trm))
dat <- dat[1:n,]
retinc <- dat[,10]
i <- as.matrix(dat[,28])
y <- as.matrix(dat[,29])
dat <- dat[,1:27]

dat[,23] <- dat[,23]-dat[,20]-dat[,21]-dat[,22]
xx <- cbind((retinc>0),retinc,dat[,12:17])
region <- as.matrix(dat[,27])
r <- unique(region)
r <- as.matrix(sort(r))
xreg <- (region%*%matrix(1,1,nrow(r))==matrix(1,nrow(region),1)%*%t(r))
xx <- cbind(xx,dat[,18:26],xreg)
mx <- matrix(1,n,1)%*%t(as.matrix(colMeans(xx)))
if (noinc==1){
  i0 <- (i==0)
  xx <- cbind(i0,xx)
}

rm(dat)
q <- as.matrix(i[floor(seq(per,per+(qn-1)*(1-per*2)/qn,(1-per*2)/qn)*n)])
out <- test2(y,xx,i,q)
f  <- out$f
yp <- out$yp
e  <- out$et  

#*************************************************#
# Bootstrap #

boot <- matrix(0,rep,1) 
for (r in 1:rep){
  yb <- yp+e[ceiling(runif(n)*n)]
  out <- test2(yb,xx,i,q)
  boot[r] <- out$f
}
pv <- mean(boot>=f)
cat (sample,"households","\n")
cat ("Trimmed Top Quantile of Income ", trm, "\n")
cat ("Bootstrap Replications         ", rep, "\n")
cat ("Number of Quantiles            ", qn, "\n")
cat ("F Tests for Second Knot","\n")
cat ("\n")
cat ("     ","F-Test","     ","P-Value", "\n")
cat ("     ",format(f,digits=6) ,"     ",pv,"\n")
cat ("\n")
cat ("\n")
cat ("\n")

#*************************************************#

smp <- smp+1
}



